# -*- coding: utf-8 -*-
# by einfall, pclin & Sven_H for DreamOS

from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox
from Components.ActionMap import ActionMap
from Components.Sources.List import List
from Components.MenuList import MenuList
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Components.MultiContent import MultiContentEntryText, MultiContentEntryTextAlphaBlend
from Components.Pixmap import Pixmap
from Components.ConfigList import ConfigList, ConfigListScreen
from Components.config import config, getConfigListEntry, ConfigText, ConfigSelectionNumber, ConfigSubsection, NoSave
from enigma import gFont, ePoint, eSize, getDesktop, eListboxPythonMultiContent, RT_HALIGN_CENTER, RT_VALIGN_CENTER, RT_WRAP, eListbox, eTimer
from skin import TemplatedListFonts, parseColor
import subprocess
import threading, re, os, json
from Screens.Console import Console
from Components.FileList import FileList

sz_w = getDesktop(0).size().width()

hyperionremote_sh = "systemctl"
hyperioncontrol_version = "1.1"

json.encoder.FLOAT_REPR = lambda o: format(o, '.4f')

#== language-support ====================================
from Components.Language import language
from os import environ as os_environ 
import gettext
from Tools.Directories import resolveFilename, SCOPE_LANGUAGE, SCOPE_PLUGINS
lang = language.getLanguage()
os_environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("enigma2-plugins", resolveFilename(SCOPE_LANGUAGE))
gettext.bindtextdomain("HyperionControl", "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/HyperionControl/locale"))

def _(txt):
	t = gettext.dgettext("HyperionControl", txt)
	if t == txt:
		t = gettext.gettext(txt)
	if t == txt:
		t = gettext.dgettext("enigma2-plugins", txt)
	return t
#== end language support ================================

class hyperionControlColors(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlColors" position="center,100" size="900,900" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,820" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,820" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,820"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,820" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,820" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,820" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;30" foregroundColor="white" position="680,820" size="200,70" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,800" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="850,780" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""

	else:
		skin = """
		<screen name="hyperionControlColors" position="center,80" size="590,620" transparent="0">
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget source="key_yellow" render="Label" font="Regular;22" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="440,570" size="140,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,3" size="575,540" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions',], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'previousSection' : self.pageUp,
		'nextSection' : self.pageDown,
		})

		tlf = TemplatedListFonts()
		#english colornames
		self.litemslist_en = [(('Maroon', '#800000'),), (('Dark Red', '#8B0000'),), (('Brown', '#A52A2A'),), (('Firebrick', '#B22222'),), (('Crimson', '#DC143C'),), (('Red', '#FF0000'),), (('Tomato', '#FF6347'),), (('Coral', '#FF7F50'),), (('Indian Red', '#CD5C5C'),), (('Light Coral', '#F08080'),), (('Dark Salmon', '#E9967A'),), (('Salmon', '#FA8072'),), (('Light Salmon', '#FFA07A'),), (('Orange Red', '#FF4500'),), (('Dark Orange', '#FF8C00'),), (('Orange', '#FFA500'),), (('Gold', '#FFD700'),), (('Dark Golden Rod', '#B8860B'),), (('Golden Rod', '#DAA520'),), (('Pale Golden rod', '#EEE8AA'),), (('Dark Khaki', '#BDB76B'),), (('Khaki', '#F0E68C'),), (('Olive', '#808000'),), (('Yellow', '#FFFF00'),), (('Yellow Green', '#9ACD32'),), (('Dark Olive Green', '#556B2F'),), (('Olive Drab', '#6B8E23'),), (('Lawn Green', '#7CFC00'),), (('Chart Reuse', '#7FFF00'),), (('Green Yellow', '#ADFF2F'),), (('Dark Green', '#006400'),), (('Green', '#008000'),), (('Forest Green', '#228B22'),), (('Lime', '#00FF00'),), (('Lime Green', '#32CD32'),), (('Light Green', '#90EE90'),), (('Pale Green', '#98FB98'),), (('Dark Sea Green', '#8FBC8F'),), (('Medium Spring Green', '#00FA9A'),), (('Spring Green', '#00FF7F'),), (('Sea Green', '#2E8B57'),), (('Medium Aqua Marine', '#66CDAA'),), (('Medium Sea Green', '#3CB371'),), (('Light Sea Green', '#20B2AA'),), (('Dark Slate Gray', '#2F4F4F'),), (('Teal', '#008080'),), (('Dark Cyan', '#008B8B'),), (('Cyan', '#00FFFF'),), (('Light Cyan', '#E0FFFF'),), (('Dark Turquoise', '#00CED1'),), (('Turquoise', '#40E0D0'),), (('Medium Turquoise', '#48D1CC'),), (('Pale Turquoise', '#AFEEEE'),), (('Aqua Marine', '#7FFFD4'),), (('Powder Blue', '#B0E0E6'),), (('Cadet Blue', '#5F9EA0'),), (('Steel Blue', '#4682B4'),), (('Corn Flower Blue', '#6495ED'),), (('Deep Sky Blue', '#00BFFF'),), (('Dodger Blue', '#1E90FF'),), (('Light Blue', '#ADD8E6'),), (('Sky Blue', '#87CEEB'),), (('Light Sky Blue', '#87CEFA'),), (('Midnight Blue', '#191970'),), (('Navy', '#000080'),), (('Dark Blue', '#00008B'),), (('Medium Blue', '#0000CD'),), (('Blue', '#0000FF'),), (('Royal Blue', '#4169E1'),), (('Blue Violet', '#8A2BE2'),), (('Indigo', '#4B0082'),), (('Dark Slate Blue', '#483D8B'),), (('Slate Blue', '#6A5ACD'),), (('Medium Slate Blue', '#7B68EE'),), (('Medium Purple', '#9370DB'),), (('Dark Magenta', '#8B008B'),), (('Dark Violet', '#9400D3'),), (('Dark Orchid', '#9932CC'),), (('Medium Orchid', '#BA55D3'),), (('Purple', '#800080'),), (('Thistle', '#D8BFD8'),), (('Plum', '#DDA0DD'),), (('Violet', '#EE82EE'),), (('Magenta / Fuchsia', '#FF00FF'),), (('Orchid', '#DA70D6'),), (('Medium Violet Red', '#C71585'),), (('Pale Violet Red', '#DB7093'),), (('Deep Pink', '#FF1493'),), (('Hot Pink', '#FF69B4'),), (('Light Pink', '#FFB6C1'),), (('Pink', '#FFC0CB'),), (('Antique White', '#FAEBD7'),), (('Beige', '#F5F5DC'),), (('Bisque', '#FFE4C4'),), (('Blanched Almond', '#FFEBCD'),), (('Wheat', '#F5DEB3'),), (('Corn Silk', '#FFF8DC'),), (('Lemon Chiffon', '#FFFACD'),), (('Light Golden Rod Yellow', '#FAFAD2'),), (('Light Yellow', '#FFFFE0'),), (('Saddle Brown', '#8B4513'),), (('Sienna', '#A0522D'),), (('Chocolate', '#D2691E'),), (('Peru', '#CD853F'),), (('Sandy Brown', '#F4A460'),), (('Burly Wood', '#DEB887'),), (('Tan', '#D2B48C'),), (('Rosy Brown', '#BC8F8F'),), (('Moccasin', '#FFE4B5'),), (('Navajo White', '#FFDEAD'),), (('Peach Puff', '#FFDAB9'),), (('Misty Rose', '#FFE4E1'),), (('Lavender Blush', '#FFF0F5'),), (('Linen', '#FAF0E6'),), (('Old Lace', '#FDF5E6'),), (('Papaya Whip', '#FFEFD5'),), (('Sea Shell', '#FFF5EE'),), (('Mint Cream', '#F5FFFA'),), (('Slate Gray', '#708090'),), (('Light Slate Gray', '#778899'),), (('Light Steel Blue', '#B0C4DE'),), (('Lavender', '#E6E6FA'),), (('Floral White', '#FFFAF0'),), (('Alice Blue', '#F0F8FF'),), (('Ghost White', '#F8F8FF'),), (('Honeydew', '#F0FFF0'),), (('Ivory', '#FFFFF0'),), (('Azure', '#F0FFFF'),), (('Snow', '#FFFAFA'),), (('Dim Gray', '#696969'),), (('Gray', '#808080'),), (('Dark gray', '#A9A9A9'),), (('Silver', '#C0C0C0'),), (('Light Gray', '#D3D3D3'),), (('Gainsboro', '#DCDCDC'),), (('White Smoke', '#F5F5F5'),), (('White', '#FFFFFF'),)]
		
		#german colornames
		self.litemslist_de = [(('Kastanienbraun', '#800000'),), (('Dunkelrot', '#8B0000'),), (('Braun', '#A52A2A'),), (('Feuerrot', '#B22222'),), (('Purpur', '#DC143C'),), (('Rot', '#FF0000'),), (('Tomatenrot', '#FF6347'),), (('Korallrot', '#FF7F50'),), (('Indianerrot', '#CD5C5C'),), (('Helles Korallrot', '#F08080'),), (('Dunkles Lachsrosa', '#E9967A'),), (('Lachsrosa', '#FA8072'),), (('Helles Lachsrosa', '#FFA07A'),), (('Orangerot', '#FF4500'),), (('Dunkles Orangerot', '#FF8C00'),), (('Orangerot', '#FFA500'),), (('Gold', '#FFD700'),), (('Dunkle Goldrute', '#B8860B'),), (('Goldrute', '#DAA520'),), (('Blaß Goldrute', '#EEE8AA'),), (('Dunkler Erdton', '#BDB76B'),), (('Erdton', '#F0E68C'),), (('Olive', '#808000'),), (('Gelb', '#FFFF00'),), (('Gelbgrün', '#9ACD32'),), (('Dunkles Olivegrün', '#556B2F'),), (('Graugrün', '#6B8E23'),), (('Grasgrün', '#7CFC00'),), (('Chartreuse', '#7FFF00'),), (('Grüngelb', '#ADFF2F'),), (('Dunkelgrün', '#006400'),), (('Grün', '#008000'),), (('Waldgrün', '#228B22'),), (('Limone', '#00FF00'),), (('Limonengrün', '#32CD32'),), (('Hellgrün', '#90EE90'),), (('Blaßgrün', '#98FB98'),), (('Dunkel Seegrün', '#8FBC8F'),), (('Mittelfrühlingsgrün', '#00FA9A'),), (('Frühlingsgrün', '#00FF7F'),), (('Meergrün', '#2E8B57'),), (('Mittelaquamarinblau', '#66CDAA'),), (('Mittelseegrün', '#3CB371'),), (('Hellmeergrün', '#20B2AA'),), (('Dunkel Schiefergrau', '#2F4F4F'),), (('Blaugrün', '#008080'),), (('Dunkles Zyan', '#008B8B'),), (('Zyan', '#00FFFF'),), (('Helles Zyan', '#E0FFFF'),), (('Dunkeltürkis', '#00CED1'),), (('Türkis', '#40E0D0'),), (('Mitteltürkis', '#48D1CC'),), (('Blaßtürkis', '#AFEEEE'),), (('Aquamarin', '#7FFFD4'),), (('Pulverblau', '#B0E0E6'),), (('Kadettenblau', '#5F9EA0'),), (('Stahlblau', '#4682B4'),), (('Kornblumenblau', '#6495ED'),), (('Dunkelhimmelblau', '#00BFFF'),), (('Persenningblau', '#1E90FF'),), (('Hellblau', '#ADD8E6'),), (('Himmelblau', '#87CEEB'),), (('Hellhimmelblau', '#87CEFA'),), (('Mitternachtsblau', '#191970'),), (('Navy', '#000080'),), (('Dunkelblau', '#00008B'),), (('Mittelblau', '#0000CD'),), (('Blau', '#0000FF'),), (('Königsblau', '#4169E1'),), (('Blauviolett', '#8A2BE2'),), (('Indigoblau', '#4B0082'),), (('Dunkles Schieferblau', '#483D8B'),), (('Schlieferblau', '#6A5ACD'),), (('Mittelschieferblau', '#7B68EE'),), (('Mittellila', '#9370DB'),), (('Dunkelmagenta', '#8B008B'),), (('Dunkelviolett', '#9400D3'),), (('Dunkel Orchidee', '#9932CC'),), (('Mittelorchidee', '#BA55D3'),), (('Lila', '#800080'),), (('Distel', '#D8BFD8'),), (('Pflaume', '#DDA0DD'),), (('Violett', '#EE82EE'),), (('Magenta', '#FF00FF'),), (('Orchidee', '#DA70D6'),), (('Mittelviolettrot', '#C71585'),), (('Blaßviolettrot', '#DB7093'),), (('Dunkelrosa', '#FF1493'),), (('Knallrosa', '#FF69B4'),), (('Hellrosa', '#FFB6C1'),), (('Rosa', '#FFC0CB'),), (('Antikweiß', '#FAEBD7'),), (('Beige', '#F5F5DC'),), (('Bisquit', '#FFE4C4'),), (('Mandelweiß', '#FFEBCD'),), (('Weizen', '#F5DEB3'),), (('Kornseide', '#FFF8DC'),), (('Zitronenchiffon', '#FFFACD'),), (('Hellgoldrute Gelb', '#FAFAD2'),), (('Hellgelb', '#FFFFE0'),), (('Sattelbraun', '#8B4513'),), (('Sienna', '#A0522D'),), (('Schokolade', '#D2691E'),), (('Peru', '#CD853F'),), (('Sandbraun', '#F4A460'),), (('Burlywood', '#DEB887'),), (('Hautfarben', '#D2B48C'),), (('Rosigbraun', '#BC8F8F'),), (('Mokassin', '#FFE4B5'),), (('Navajo Weiß', '#FFDEAD'),), (('Pfirsich', '#FFDAB9'),), (('Blaßrose', '#FFE4E1'),), (('Lavendelhauch', '#FFF0F5'),), (('Leinen', '#FAF0E6'),), (('Alte Spitze', '#FDF5E6'),), (('Papaya Creme', '#FFEFD5'),), (('Muschelweiß', '#FFF5EE'),), (('Pfefferminzcreme', '#F5FFFA'),), (('Schiefergrau', '#708090'),), (('Hellschiefergrau', '#778899'),), (('Hellstahlblau', '#B0C4DE'),), (('Lavendel', '#E6E6FA'),), (('Blütenweiß', '#FFFAF0'),), (('Aliceblau', '#F0F8FF'),), (('Geisterweiß', '#F8F8FF'),), (('Honigtau', '#F0FFF0'),), (('Elfenbein', '#FFFFF0'),), (('Azur', '#F0FFFF'),), (('Schneeweiß', '#FFFAFA'),), (('Blaßgrau', '#696969'),), (('Grau', '#808080'),), (('Dunkelgrau', '#A9A9A9'),), (('Silber', '#C0C0C0'),), (('Hellgrau', '#D3D3D3'),), (('Gainsboro', '#DCDCDC'),), (('Rauch', '#F5F5F5'),), (('Weiß', '#FFFFFF'),)]
		
		self.litemsverlauf = [(('','#800000'),), (('','#8D0000'),), (('','#990000'),), (('','#A60000'),), (('','#B30000'),), (('','#C00000'),), (('','#CC0000'),), (('','#D90000'),), (('','#E60000'),), (('','#F20000'),), (('','#FF0000'),), (('','#FF1010'),), (('','#FF2020'),), (('','#FF3030'),), (('','#FF3F3F'),), (('','#FF4F4F'),), (('','#FF5F5F'),), (('','#FF6F6F'),), (('','#FF7F7F'),), (('','#FF8F8F'),), (('','#FF9F9F'),), (('','#FFAFAF'),), (('','#FFBEBE'),), (('','#FFCECE'),), (('','#FFDEDE'),), (('','#FFEEEE'),), (('','#008000'),), (('','#008D00'),), (('','#009900'),), (('','#00A600'),), (('','#00B300'),), (('','#00C000'),), (('','#00CC00'),), (('','#00D900'),), (('','#00E600'),), (('','#00F200'),), (('','#00FF00'),), (('','#10FF10'),), (('','#20FF20'),), (('','#30FF30'),), (('','#3FFF3F'),), (('','#4FFF4F'),), (('','#5FFF5F'),), (('','#6FFF6F'),), (('','#7FFF7F'),), (('','#8FFF8F'),), (('','#9FFF9F'),), (('','#AFFFAF'),), (('','#BEFFBE'),), (('','#CEFFCE'),), (('','#DEFFDE'),), (('','#EEFFEE'),), (('','#000080'),), (('','#00008D'),), (('','#000099'),), (('','#0000A6'),), (('','#0000B3'),), (('','#0000C0'),), (('','#0000CC'),), (('','#0000D9'),), (('','#0000E6'),), (('','#0000F2'),), (('','#0000FF'),), (('','#1010FF'),), (('','#2020FF'),), (('','#3030FF'),), (('','#3F3FFF'),), (('','#4F4FFF'),), (('','#5F5FFF'),), (('','#6F6FFF'),), (('','#7F7FFF'),), (('','#8F8FFF'),), (('','#9F9FFF'),), (('','#AFAFFF'),), (('','#BEBEFF'),), (('','#CECEFF'),), (('','#DEDEFF'),), (('','#EEEEFF'),), (('','#ADAD00'),), (('','#B5B500'),), (('','#BDBD00'),), (('','#C6C600'),), (('','#CECE00'),), (('','#D6D600'),), (('','#DEDE00'),), (('','#E6E600'),), (('','#EFEF00'),), (('','#F7F700'),), (('','#FFFF00'),), (('','#FFFF10'),), (('','#FFFF20'),), (('','#FFFF30'),), (('','#FFFF3F'),), (('','#FFFF4F'),), (('','#FFFF5F'),), (('','#FFFF6F'),), (('','#FFFF7F'),), (('','#FFFF8F'),), (('','#FFFF9F'),), (('','#FFFFAF'),), (('','#FFFFBE'),), (('','#FFFFCE'),), (('','#FFFFDE'),), (('','#FFFFEE'),), (('','#00ADAD'),), (('','#00B5B5'),), (('','#00BDBD'),), (('','#00C6C6'),), (('','#00CECE'),), (('','#00D6D6'),), (('','#00DEDE'),), (('','#00E6E6'),), (('','#00EFEF'),), (('','#00F7F7'),), (('','#00FFFF'),), (('','#10FFFF'),), (('','#20FFFF'),), (('','#30FFFF'),), (('','#3FFFFF'),), (('','#4FFFFF'),), (('','#5FFFFF'),), (('','#6FFFFF'),), (('','#7FFFFF'),), (('','#8FFFFF'),), (('','#9FFFFF'),), (('','#AFFFFF'),), (('','#BEFFFF'),), (('','#CEFFFF'),), (('','#DEFFFF'),), (('','#EEFFFF'),), (('','#6D006D'),), (('','#7C007C'),), (('','#8A008A'),), (('','#990099'),), (('','#A700A7'),), (('','#B600B6'),), (('','#C500C5'),), (('','#D300D3'),), (('','#E200E2'),), (('','#FF00FF'),), (('','#FF10FF'),), (('','#FF20FF'),), (('','#FF30FF'),), (('','#FF3FFF'),), (('','#FF4FFF'),), (('','#FF5FFF'),), (('','#FF6FFF'),), (('','#FF7FFF'),), (('','#FF8FFF'),), (('','#FF9FFF'),), (('','#FFAFFF'),), (('','#FFBEFF'),), (('','#FFCEFF'),), (('','#FFDEFF'),), (('','#FFEEFF'),), (('','#3F1F09'),), (('','#47230A'),), (('','#4E270B'),), (('','#562A0C'),), (('','#5D2E0D'),), (('','#65320E'),), (('','#6D360F'),), (('','#743A10'),), (('','#7C3D11'),), (('','#834112'),), (('','#8B4513'),), (('','#924F1F'),), (('','#99592B'),), (('','#A16337'),), (('','#A86D42'),), (('','#AF774E'),), (('','#B6815A'),), (('','#BD8B66'),), (('','#C59472'),), (('','#CC9E7E'),), (('','#D3A88A'),), (('','#DAB296'),), (('','#E1BCA1'),), (('','#E9C6AD'),), (('','#F0D0B9'),), (('','#F7DAC5'),), (('','#E09100'),), (('','#E39300'),), (('','#E69500'),), (('','#E99700'),), (('','#EC9900'),), (('','#F09B00'),), (('','#F39D00'),), (('','#F69F00'),), (('','#F9A100'),), (('','#FCA300'),), (('','#FFA500'),), (('','#FFAC14'),), (('','#FFB329'),), (('','#FFBB3D'),), (('','#FFC252'),), (('','#FFC966'),), (('','#FFD07A'),), (('','#FFD78F'),), (('','#FFDFA3'),), (('','#FFE6B8'),), (('','#FFEDCC'),), (('','#111111'),), (('','#1C1C1C'),), (('','#272727'),), (('','#323232'),), (('','#3D3D3D'),), (('','#484848'),), (('','#535353'),), (('','#5E5E5E'),), (('','#696969'),), (('','#747474'),), (('','#808080'),), (('','#8B8B8B'),), (('','#969696'),), (('','#A1A1A1'),), (('','#ACACAC'),), (('','#B7B7B7'),), (('','#C2C2C2'),), (('','#CDCDCD'),), (('','#D8D8D8'),), (('','#E3E3E3'),), (('','#EEEEEE'),)]
		
		if lang == "de_DE":
			self.litems = self.litemslist_de
		else:
			self.litems = self.litemslist_en
		self.ColorPage = "farbliste"
		self["key_yellow"] = StaticText(_("gradient"))
		self["key_blue"] = StaticText(_("Image Mode"))
		
		if sz_w == 1920:
			self.itemHeight = 60
			self.itemWidth = 850
			self.ml = MenuList(self.litems, content=eListboxPythonMultiContent, itemSize=self.itemHeight, margin=ePoint(0,0), selectionZoom=1.1)
		else:
			self.itemHeight = 45
			self.itemWidth = 575
			self.ml = MenuList(self.litems, content=eListboxPythonMultiContent, itemSize=self.itemHeight, margin=ePoint(0,0), selectionZoom=1.1)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.SMALL), tlf.size(tlf.SMALL)))
		self.ml.l.setFont(1, gFont(tlf.face(tlf.MEDIUM), tlf.size(tlf.MEDIUM)))
		self.ml.l.setBuildFunc(self.showList, True)
		self['list'] = self.ml
		self.onLayoutFinish.append(self.buildList)
		self.setTitle(_("Hyperion Control - Colors"))
		
	def buildList(self):
		self["list"].setList(self.litems)

	def pageUp(self):
		self["list"].pageUp()

	def pageDown(self):
		self["list"].pageDown()
		
	def showList(self, item, selected):
		(text, hcol) = item
		col = parseColor(hcol).argb()
		if selected:
			textcol = parseColor("#000000").argb()
			if col == textcol:
				textcol = parseColor("#FFFFFF").argb()
		else:
			textcol = parseColor("#FFFFFF").argb()
			if col == textcol:
				textcol = parseColor("#000000").argb()
		res = [item]
		if sz_w == 1920:
			margin = 5
			pos = (margin+20,margin)
			EntrySize = (self.itemWidth - margin*2 -30,self.itemHeight-margin*2)
		else:
			margin = 4
			pos = (margin+15,margin)
			EntrySize = (self.itemWidth - margin*2 -20,self.itemHeight-margin*2)
		try:
			if selected:
				res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=1, text=text, flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER|RT_WRAP, color=textcol, color_sel=textcol, backcolor=col, backcolor_sel=col))
			else:
				res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=0, text=text, flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER|RT_WRAP, color=textcol, color_sel=textcol, backcolor=col, backcolor_sel=col))
			return res
		except:
			import traceback
			traceback.print_exc()

			return None

	def key_red(self):
		self.key_exit()

	def key_green(self):
		pass
		
	def key_yellow(self):
		if self.ColorPage == "farbverlauf":
			self.ColorPage = "farbliste"
			self["key_yellow"].setText(_("gradient"))
			if lang == "de_DE":
				self.litems = self.litemslist_de
			else:
				self.litems = self.litemslist_en
		else:
			self.ColorPage = "farbverlauf"
			self["key_yellow"].setText(_("color list"))
			self.litems = self.litemsverlauf
		self.buildList()

	def key_blue(self):
		try:
			if config.plugins.hyperioncontrol.lastpicpath.value:
				path = config.plugins.hyperioncontrol.lastpicpath.value
			else:
				path = "/"
			self.session.openWithCallback(self.selectPictureCallback, HyperionControlPictureBrowser, path)
		except:
			import traceback
			traceback.print_exc()

	def selectPictureCallback(self, retvalue):
		if retvalue:
			cmd = "hyperion-remote --clearall && hyperion-remote -i " + retvalue
			print "[HyperionControl] cmd", cmd
			command = Command(cmd)
			command.run(timeout=3)
			print "[HyperionControl] cmd_out:", command.out

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		(name, color) = self['list'].getCurrent()[0]
		print name, color
		cmd = "-c '%s'" % color
		send_CMD(cmd)

	def key_exit(self):
		self.close()
		

#to get consolen-output with timeout-function
class Command(object):
	def __init__(self, cmd):
		self.cmd = cmd
		self.process = None

	def run(self, timeout):
		def target():
			self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
			(self.out,self.err) = self.process.communicate()

		thread = threading.Thread(target=target)
		thread.start()
		self.timeout=False

		thread.join(float(timeout))
		if thread.is_alive():
			self.process.terminate()
			self.timeout=True
			self.process = None
			thread = None
			return
#=============================================

class ConfigListHC(ConfigList):
	def __init__(self, list, session = None):
		ConfigList.__init__(self, list, session = session)
	
	def jumpToPreviousSection(self):
		index = self.getCurrentIndex() - 1
		index = self.getCurrentIndex() - 1
		maxlen = len(self._ConfigList__list)
		while index >= 0 and maxlen > 0:
			index -= 1
			#fix jump to PreviousSection on empty lines in ConfigList
			if index in self._headers and self.list[index] != ("",):
				if index + 1 < maxlen:
					self.setCurrentIndex(index + 1)
					return
				else:
					self.setCurrentIndex(index - 1)
					return
		self.pageUp()

#=========================================================================

class hyperionControlSetup(Screen, ConfigListScreen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlSetup" position="center,100" size="900,900" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,820" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,820" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,820"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,820" size="200,70" scale="stretch"/>
			<eLabel name="" position="10,800" size="880,2" backgroundColor="grey" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,820" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="240,820" size="200,70" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,820" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;30" foregroundColor="white" position="680,820" size="200,70" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="config" seperation="100" position="10,10" size="880,770" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlSetup" position="center,80" size="590,620">
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="150,570" size="140,40" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
			<widget source="key_yellow" render="Label" font="Regular;22" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="440,570" size="140,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="config" position="10,10" seperation="100" size="570,545" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
		
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions'], {
		'cancel': self.key_exit,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'menu'	: self.menu,
		})
		
		self["key_yellow"] = StaticText()
		self["key_blue"] = StaticText(_("Options"))
		self.setupPage = "transform"
		
		self.list = []
		ConfigListScreen.__init__(self, self.list, session = session, on_change = self.changed)
		self["config"] = ConfigListHC(self.list, session = session)
		self.buildConfig()
		self.setTitle(_("Hyperion Control - Setup"))
		self["key_yellow"].setText(_("Device/LED"))
		self.onShown.append(self.readData)
		self.lastLEDcountTxt = None
	
	def readData(self):
		self.onShown.remove(self.readData)
		self.readValuesFromConfig()
		readValuesFromRemote(self)
	
	def saveJson(self):
		try:
			# hsv
			self.jsonConfig['color']['transform'][0]['hsv']['saturationGain'] = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsv']['valueGain'] = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0
			
			# hsl
			self.jsonConfig['color']['transform'][0]['hsl']['saturationLGain'] = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsl']['luminanceGain'] = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
			self.jsonConfig['color']['transform'][0]['hsl']['luminanceMinimum'] = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0
			
			# red
			self.jsonConfig['color']['transform'][0]['red']['gamma'] = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
			self.jsonConfig['color']['transform'][0]['red']['threshold'] = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['red'] = int(config.plugins.hyperioncontrol.temperatureRed.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['redChannel'] = int(config.plugins.hyperioncontrol.redAdjustRed.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['redChannel'] = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['redChannel'] = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
			self.jsonConfig['color']['transform'][0]['red']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
			self.jsonConfig['color']['transform'][0]['red']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
			
			# green
			self.jsonConfig['color']['transform'][0]['green']['gamma'] = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
			self.jsonConfig['color']['transform'][0]['green']['threshold'] = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['green'] = int(config.plugins.hyperioncontrol.temperatureGreen.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['greenChannel'] = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['greenChannel'] = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['greenChannel'] = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
			self.jsonConfig['color']['transform'][0]['green']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
			self.jsonConfig['color']['transform'][0]['green']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
			
			# blue
			self.jsonConfig['color']['transform'][0]['blue']['gamma'] = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
			self.jsonConfig['color']['transform'][0]['blue']['threshold'] = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
			self.jsonConfig['color']['temperature'][0]['correctionValues']['blue'] = int(config.plugins.hyperioncontrol.temperatureBlue.value) 
			self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['blueChannel'] = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['blueChannel'] = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
			self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['blueChannel'] = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
			self.jsonConfig['color']['transform'][0]['blue']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
			self.jsonConfig['color']['transform'][0]['blue']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
			
			#Framegrabber
			self.jsonConfig['framegrabber']['width'] = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[0])
			self.jsonConfig['framegrabber']['height'] = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[1])
			self.jsonConfig['framegrabber']['frequency_Hz'] = int(config.plugins.hyperioncontrol.framegrabberFreq.value) * 1.0
			
			#BlackborderDetector
			self.jsonConfig['blackborderdetector']['enable'] = str(config.plugins.hyperioncontrol.bbdEnable.value).lower()
			self.jsonConfig['blackborderdetector']['threshold'] = int(config.plugins.hyperioncontrol.bbdThreshold.value) / 100.0
			
			# device
			self.jsonConfig['device']['type'] = str(config.plugins.hyperioncontrol.deviceType.value)
			self.jsonConfig['device']['name'] = str(config.plugins.hyperioncontrol.configName.value)
			if config.plugins.hyperioncontrol.deviceType.value in ("sedu","adalight","atmo"):
				self.jsonConfig['device']['output'] = "/dev/ttyS0"
				self.jsonConfig['device']['delayafterconnect'] = int(config.plugins.hyperioncontrol.delayafterconnect.value)
			else:
				if config.plugins.hyperioncontrol.deviceType.value == "philipshue":
					self.jsonConfig['device']['output'] = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.outputIP.value])
				elif config.plugins.hyperioncontrol.deviceType.value == "udpraw":
					self.jsonConfig['device']['output'] = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.outputIP.value]) + ":19446"
			self.jsonConfig['device']['colorOrder'] = str(config.plugins.hyperioncontrol.colorOrder.value)
			self.jsonConfig['device']['rate'] = int(config.plugins.hyperioncontrol.baudrate.value)
			
			#LED-Config
			if config.plugins.hyperioncontrol.ledBegin.value != "0":
				LEDconfig = self.createNewLEDConfig()
				if LEDconfig:
					self.jsonConfig['leds'] = LEDconfig
				else:
					self.session.open(MessageBox,_("Error when generating the LED config!"), MessageBox.TYPE_INFO,timeout=5)
			
			#aml-Service
			self.writeAmlService()
			
			config_file_path ="/etc/hyperion/hyperion.config.json"
			
			if os.path.exists(config_file_path): 
				#write values in config.json
				with open(config_file_path, "w") as write_file:
					json.dump(self.jsonConfig, write_file, ensure_ascii=False, indent=4)
				#daemon reload und services neu starten
				cmd = "systemctl daemon-reload && systemctl restart hyperion && systemctl restart hyperionaml"
				os.system(cmd)
				self.session.open(MessageBox,_("The settings were saved in '/etc/hyperion/hyperion.config.json'.\n\nHyperion and Hyperionaml were restarted."), MessageBox.TYPE_INFO,timeout=10)
			else:
				self.session.open(MessageBox,_("File '/etc/hyperion/hyperion.config.json.bak' not found!"), MessageBox.TYPE_INFO,timeout=5)
			
			config.plugins.hyperioncontrol.save()
			
			
		
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,_("When saving the config.json an error has occurred!"), MessageBox.TYPE_INFO,timeout=5)

	def writeAmlService(self):
		
		script_txt = "[Unit]\n"
		script_txt += "Description=Hyperionaml Systemd service\n"
		script_txt += "Wants=enigma2.service\n"
		script_txt += "Wants=hyperion.service\n"
		script_txt += "[Service]\n"
		script_txt += "Type=simple\n"
		script_txt += "User=root\n"
		script_txt += "Group=root\n"
		script_txt += "UMask=007\n"
		
		width = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[0])
		height = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[1])
		freq = config.plugins.hyperioncontrol.framegrabberFreq.value
		script_txt += "ExecStart=/usr/bin/hyperion-aml -f %s --width %s --height %s\n" % (freq, width, height)
		
		script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
		script_txt += "Restart=on-failure\n"
		script_txt += "TimeoutStopSec=10\n"
		script_txt += "[Install]\n"
		script_txt += "WantedBy=multi-user.target\n"
		
		amlservice_file = open("/lib/systemd/system/hyperionaml.service", "w+")
		amlservice_file.write(script_txt)
		amlservice_file.close()
		os.chmod("/lib/systemd/system/hyperionaml.service",0644)

	def getNewFixScanValue(self, ledNr, changeNumber, changes, currentPosition, hscan_min, hscan_max, vscan_min, vscan_max):
		#print "== scanValue", ledNr, changes
		hPicDeep = int(config.plugins.hyperioncontrol.hPicDeep.value) / 100.0
		vPicDeep = int(config.plugins.hyperioncontrol.vPicDeep.value) / 100.0
		
		#print "== position, changeLED, ledNr, ledIndex", currentPosition, changes[changeNumber], ledNr, ledNr-1
		
		#if ledNr == 1:
		#	print "== start position '%s' on ledNr '%s', index '%s'" % (currentPosition, ledNr, ledNr-1)
		
		#reset scan_min-Values after change position
		if changeNumber >0 and ledNr == changes[changeNumber-1]+1:
			#print "== new position '%s' on ledNr '%s', index '%s'" % (currentPosition, ledNr, ledNr-1)
			if currentPosition == "bottom":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
					hscan_min = 1.0000
				elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
					hscan_min = 0.0000 - self.h_diff
			
			elif currentPosition == "top":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
					hscan_min = 0.0000 - self.h_diff
				elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
					hscan_min = 1.0000
			
			elif currentPosition == "right":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
					vscan_min = 0.0000 - self.vr_diff
				elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
					vscan_min = 1.0000
			
			elif currentPosition == "left":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
					vscan_min = 1.0000
				elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
					vscan_min = 0.0000 - self.vl_diff
		
		#set scan-Values on Corner LED
		if changeNumber < 4 and ledNr == changes[changeNumber]:
			if currentPosition == "bottom":
				if config.plugins.hyperioncontrol.ledDirection.value == "1" and config.plugins.hyperioncontrol.ledCornerBottomLeft.value: #UZ
					#print "== corner led BottomLeft", ledNr, ledNr-1
					return (0.0000, vPicDeep, 1.0000-hPicDeep, 1.0000) #fix value for corner led BottomLeft
				elif config.plugins.hyperioncontrol.ledDirection.value == "2" and config.plugins.hyperioncontrol.ledCornerBottomRight.value: #GUZ
					#print "== corner led BottomRight", ledNr, ledNr-1
					return (1.0000-vPicDeep, 1.0000, 1.0000-hPicDeep,1.0000) #fix value for corner led BottomRight
			elif currentPosition == "top":
				if config.plugins.hyperioncontrol.ledDirection.value == "1" and config.plugins.hyperioncontrol.ledCornerTopRight.value: #UZ
					#print "== corner led TopRight", ledNr, ledNr-1
					return (1.0000-vPicDeep, 1.0000, 0.0000, hPicDeep) #fix value for corner led TopRight
				elif config.plugins.hyperioncontrol.ledDirection.value == "2" and config.plugins.hyperioncontrol.ledCornerTopLeft.value: #GUZ
					#print "== corner led TopLeft", ledNr, ledNr-1
					return (0.0000, vPicDeep, 0.0000, hPicDeep) #fix value for corner LED TopLeft
			elif currentPosition == "right":
				if config.plugins.hyperioncontrol.ledDirection.value == "1" and config.plugins.hyperioncontrol.ledCornerBottomRight.value: #UZ
					#print "== corner led BottomRight", ledNr, ledNr-1
					return (1.0000-vPicDeep, 1.0000, 1.0000-hPicDeep,1.0000) #fix value for corner led BottomRight
				elif config.plugins.hyperioncontrol.ledDirection.value == "2" and config.plugins.hyperioncontrol.ledCornerTopRight.value: #GUZ
					#print "== corner led TopRight", ledNr, ledNr-1
					return (1.0000-vPicDeep, 1.0000, 0.0000, hPicDeep) #fix value for corner led TopRight
			elif currentPosition == "left":
				if config.plugins.hyperioncontrol.ledDirection.value == "1" and config.plugins.hyperioncontrol.ledCornerTopLeft.value: #UZ
					#print "== corner led TopLeft", ledNr, ledNr-1
					return (0.0000, vPicDeep, 0.0000, hPicDeep) #fix value for corner LED TopLeft
				elif config.plugins.hyperioncontrol.ledDirection.value == "2" and config.plugins.hyperioncontrol.ledCornerBottomLeft.value: #GUZ
					#print "== corner led BottomLeft", ledNr, ledNr-1
					return (0.0000, vPicDeep, 1.0000-hPicDeep, 1.0000) #fix value for corner led BottomLeft
		
		#jump hscan_min on LED-gap
		if self.gap > 0 and ledNr == self.gapLED:
			if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
				hscan_min = hscan_min - (self.h_diff * self.gap)
			elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
				hscan_min = hscan_min + (self.h_diff * self.gap)

		#set scanValues 
		if currentPosition == "bottom":
			if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
				hscan_min = hscan_min - self.h_diff
			elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
				hscan_min = hscan_min + self.h_diff
			hscan_max = hscan_min + self.h_diff
			return (hscan_min, hscan_max, 1.000 - hPicDeep, 1.0000) #vscan bottom fix
		
		elif currentPosition == "top":
			if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
				hscan_min = hscan_min + self.h_diff
			elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
				hscan_min = hscan_min - self.h_diff
			hscan_max = hscan_min + self.h_diff
			return (hscan_min, hscan_max, 0.0000, hPicDeep) #vscan top fix
		
		elif currentPosition == "left":
			if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
				vscan_min = vscan_min - self.vl_diff
			elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
				vscan_min = vscan_min + self.vl_diff
			vscan_max = vscan_min + self.vl_diff
			return (0.0000, vPicDeep, vscan_min, vscan_max) #hscan left fix
		
		elif currentPosition == "right":
			if config.plugins.hyperioncontrol.ledDirection.value == "1": #UZ
				vscan_min = vscan_min + self.vr_diff
			elif config.plugins.hyperioncontrol.ledDirection.value == "2": #GUZ
				vscan_min = vscan_min - self.vr_diff
			vscan_max = vscan_min + self.vr_diff
			return (1.0000 - vPicDeep, 1.0000, vscan_min, vscan_max) #hscan right fix
	
	def createNewLEDConfig(self):
		try:
			led_top   = int(config.plugins.hyperioncontrol.ledTop.value)
			led_right = int(config.plugins.hyperioncontrol.ledRight.value)
			led_left  = int(config.plugins.hyperioncontrol.ledLeft.value)
			self.gap  = int(config.plugins.hyperioncontrol.ledBottomGap.value)
			
			ledCornerTopLeft = 1 if config.plugins.hyperioncontrol.ledCornerTopLeft.value else 0
			ledCornerTopRight = 1 if config.plugins.hyperioncontrol.ledCornerTopRight.value else 0
			ledCornerBottomLeft = 1 if config.plugins.hyperioncontrol.ledCornerBottomLeft.value else 0
			ledCornerBottomRight = 1 if config.plugins.hyperioncontrol.ledCornerBottomRight.value else 0
			
			led_count = led_top*2 + led_right + led_left - self.gap + ledCornerTopLeft + ledCornerTopRight + ledCornerBottomLeft + ledCornerBottomRight
			#print "=== led-count", led_count
			
			self.h_diff = 1.0 / led_top
			self.vr_diff = 1.0 / led_right
			self.vl_diff = 1.0 / led_left
			
			hscan_min = 0
			hscan_max = 0
			vscan_min = 0
			vscan_max = 0
			
			self.gapLED = 0
			
			# wenn ledBegin "unten Mitte"
			if config.plugins.hyperioncontrol.ledBegin.value == "1":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = (led_top // 2) - (self.gap // 2) + ledCornerBottomLeft
					change1 = change0 + led_left + ledCornerTopLeft
					change2 = change1 + led_top + ledCornerTopRight
					change3 = change2 + led_right + ledCornerBottomRight
					positions = ["bottom","left","top","right","bottom"]
					hscan_min = 0.5000 - (self.h_diff * (self.gap // 2)) # UZ = unten mitte nach links
					self.gapLED = 0
				else: #GUZ
					change0 = (led_top // 2) - (self.gap // 2) + ledCornerBottomRight
					change1 = change0 + led_right + ledCornerTopRight
					change2 = change1 + led_top + ledCornerTopLeft
					change3 = change2 + led_left + ledCornerBottomLeft
					positions = ["bottom","right","top","left","bottom"]
					hscan_min = 0.5000 - self.h_diff + (self.h_diff * (self.gap // 2)) # GUZ = unten mitte nach rechts
					self.gapLED = 0
				changes = [change0,change1,change2,change3]
			
			#wenn ledBegin "rechts unten"
			if config.plugins.hyperioncontrol.ledBegin.value == "2":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = led_top - self.gap + ledCornerBottomLeft
					change1 = change0 + led_left + ledCornerTopLeft
					change2 = change1 + led_top + ledCornerTopRight
					change3 = change2 + led_right + ledCornerBottomRight
					hscan_min = 1.0000 # UZ = unten von rechts nach links
					positions = ["bottom","left","top","right"]
					if self.gap:
						self.gapLED = (led_top // 2) - (self.gap // 2) + 1
				else: #GUZ
					change0 = led_right + ledCornerTopRight
					change1 = change0 + led_top + ledCornerTopLeft
					change2 = change1 + led_left + ledCornerBottomLeft
					change3 = change2 + led_top - self.gap + ledCornerBottomRight
					positions = ["right","top","left","bottom"]
					vscan_min = 1.0000 # GUZ = rechte Seite nach oben
					if self.gap:
						self.gapLED = change2 + (led_top // 2) - (self.gap // 2) + 1
				changes = [change0,change1,change2,change3]
			
			#wenn ledBegin "rechts oben"
			if config.plugins.hyperioncontrol.ledBegin.value == "3":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = led_right + ledCornerBottomRight
					change1 = change0 + led_top - self.gap + ledCornerBottomLeft
					change2 = change1 + led_left + ledCornerTopLeft
					change3 = change2 + led_top + ledCornerTopRight
					vscan_min = 0.0000 - self.vr_diff # UZ = rechts von oben nach unten
					positions = ["right", "bottom","left","top"]
					if self.gap:
						self.gapLED = change0 + (led_top // 2) - (self.gap // 2) + 1
				else: #GUZ
					change0 = led_top + ledCornerTopLeft
					change1 = change0 + led_left + ledCornerBottomLeft
					change2 = change1 + led_top - self.gap + ledCornerBottomRight
					change3 = change2 + led_right + ledCornerTopRight
					positions = ["top","left","bottom","right"]
					hscan_min = 1.0000 # GUZ = oben von rechts nach links
					if self.gap:
						self.gapLED = change1 + (led_top // 2) - (self.gap // 2) + 1
				changes = [change0,change1,change2,change3]
			
			#wenn ledBegin "links oben"
			if config.plugins.hyperioncontrol.ledBegin.value == "4":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = led_top + ledCornerTopRight
					change1 = change0 + led_right + ledCornerBottomRight
					change2 = change1 + led_top - self.gap + ledCornerBottomLeft
					change3 = change2 + led_left + ledCornerTopLeft
					hscan_min = 0.0000 - self.h_diff # UZ = oben von links nach rechts
					positions = ["top", "right", "bottom","left"]
					if self.gap:
						self.gapLED = change1 + (led_top // 2) - (self.gap // 2) + 1
				else: #GUZ
					change0 = led_left + ledCornerBottomLeft
					change1 = change0 + led_top - self.gap + ledCornerBottomRight
					change2 = change1 + led_right + ledCornerTopRight
					change3 = change2 + led_top + ledCornerTopLeft
					positions = ["left","bottom","right","top"]
					vscan_min = 0.0000 - self.vl_diff # GUZ = links von oben nach unten
					if self.gap:
						self.gapLED = change0 + (led_top // 2) - (self.gap // 2) + 1
				changes = [change0,change1,change2,change3]
			
			#wenn ledBegin "links unten"
			if config.plugins.hyperioncontrol.ledBegin.value == "5":
				if config.plugins.hyperioncontrol.ledDirection.value == "1": # UZ
					change0 = led_left + ledCornerTopLeft
					change1 = change0 + led_top + ledCornerTopRight
					change2 = change1 + led_right + ledCornerBottomRight
					change3 = change2 + led_top - self.gap + ledCornerBottomLeft
					vscan_min = 1.000 # UZ = links von unten nach oben
					positions = ["left","top", "right", "bottom"]
					if self.gap:
						self.gapLED = change2 + (led_top // 2) - (self.gap // 2) + 1
				else: #GUZ
					change0 = led_top - self.gap + ledCornerBottomRight
					change1 = change0 + led_right + ledCornerTopRight
					change2 = change1 + led_top + ledCornerTopLeft
					change3 = change2 + led_left + ledCornerBottomLeft
					positions = ["bottom","right","top","left"]
					hscan_min = 0.0000 - self.h_diff # GUZ = unten von links nach rechts
					if self.gap:
						self.gapLED = (led_top // 2) - (self.gap // 2) + 1
				changes = [change0,change1,change2,change3]
				
			#print "=== changes, positions, gapLED", changes, positions, self.gapLED
			leds = []
			for i in range(1,led_count+1):
				position = 0
				for x in changes:
					if i<= x:
						break
					position +=1
				changeNumber = position
				
				(hscan_min, hscan_max, vscan_min, vscan_max) = self.getNewFixScanValue(i,changeNumber,changes,positions[position], hscan_min, hscan_max, vscan_min, vscan_max)
				
				from collections import OrderedDict
				led = OrderedDict()
				led['index'] = int(i-1)
				led['hscan'] = {'minimum': abs(hscan_min), 'maximum': abs(hscan_max)}
				led['vscan'] = {'minimum': abs(vscan_min), 'maximum': abs(vscan_max)}
				leds.append(led)
				#print "=== led dict", led
				
				#print "index: ", i-1
				#print '"hscan" : { "minimum" : %s, "maximum" : %s },' % (format(round(hscan_min,4),".4f"), format(round(hscan_max,4),".4f"))
				#print '"vscan" : { "minimum" : %s, "maximum" : %s }' % (format(round(vscan_min,4),".4f"), format(round(vscan_max,4),".4f"))
				#print "------------------------------------------------"
			#print "=== leds", leds
			return leds

		except:
			import traceback
			traceback.print_exc()
			return None

	def importJsonBak(self, retval):
		try:
			if not retval:
				return
			
			#read values from config.json.bak
			if not self.jsonConfig:
				self.session.open(MessageBox,_("There are no data in '/etc/hyperion/hyperion.config.json'!\n\nAn import is therefore not possible."), MessageBox.TYPE_INFO,timeout=5)
				return
			
			self.jsonConfigBak= ""
			config_file_path ="/etc/hyperion/hyperion.config.json.bak"
			if os.path.exists(config_file_path): 
				#json.bak einlesen
				valuesFromConfig = open(config_file_path).read()
				lines = valuesFromConfig.split("\n")
				newlines=[]
				for line in lines:
					cleanline = line.replace("\t","")
					if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
						newlines.append(line)
				valuesFromConfig = "\n".join(newlines)
				from collections import OrderedDict
				self.jsonConfigBak = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
				
				#alle Hauptkeys aus json.bak in json übernehmen
				for (k, v) in self.jsonConfigBak.items():
					#print "===", k
					key = str(k)
					if key in self.jsonConfig:
						self.jsonConfig[key] = self.jsonConfigBak[key]
				
				#reload configs from jsonConfigBak
				# hsv
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, self.jsonConfigBak, ['color','transform',0,'hsv','saturationGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, self.jsonConfigBak, ['color','transform',0,'hsv','valueGain'],100)
				# hsl
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, self.jsonConfigBak, ['color','transform',0,'hsl','saturationLGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, self.jsonConfigBak, ['color','transform',0,'hsl','luminanceGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, self.jsonConfigBak, ['color','transform',0,'hsl','luminanceMinimum'],100)
				# red
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, self.jsonConfigBak, ['color','transform',0,'red','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, self.jsonConfigBak, ['color','transform',0,'red','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, self.jsonConfigBak, ['color','temperature',0,'correctionValues','red'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','redChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, self.jsonConfigBak, ['color','transform',0,'red','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, self.jsonConfigBak, ['color','transform',0,'red','blacklevel'],100)
				
				# green
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, self.jsonConfigBak, ['color','transform',0,'green','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, self.jsonConfigBak, ['color','transform',0,'green','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, self.jsonConfigBak, ['color','temperature',0,'correctionValues','green'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','greenChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, self.jsonConfigBak, ['color','transform',0,'green','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, self.jsonConfigBak, ['color','transform',0,'green','blacklevel'],100)
				
				# blue
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, self.jsonConfigBak, ['color','transform',0,'blue','gamma'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, self.jsonConfigBak, ['color','transform',0,'blue','threshold'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, self.jsonConfigBak, ['color','temperature',0,'correctionValues','blue'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureRed','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureGreen','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, self.jsonConfigBak, ['color','channelAdjustment',0,'pureBlue','blueChannel'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, self.jsonConfigBak, ['color','transform',0,'blue','whitelevel'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, self.jsonConfigBak, ['color','transform',0,'blue','blacklevel'],100)
				
				# device
				setConfigValueFromJson(config.plugins.hyperioncontrol.deviceType, self.jsonConfigBak, ['device','type'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.configName, self.jsonConfigBak, ['device','name'])
				if config.plugins.hyperioncontrol.deviceType.value in ("udpraw","philipshue"):
					outputIP = setConfigValueFromJson(config.plugins.hyperioncontrol.outputIP, self.jsonConfigBak, ['device','output'], onlyReturnValue=True)
					outputIP = outputIP.replace(":19446","").split(".")
					outputIP = [int(d) for d in outputIP]
					config.plugins.hyperioncontrol.outputIP.value = outputIP
				setConfigValueFromJson(config.plugins.hyperioncontrol.colorOrder, self.jsonConfigBak, ['device','colorOrder'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.delayafterconnect, self.jsonConfigBak, ['device','delayafterconnect'])
				setConfigValueFromJson(config.plugins.hyperioncontrol.baudrate, self.jsonConfigBak, ['device','rate'],calculate=1)
				
				#Framegrabber
				width = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber','width'],None,True,"160")
				height = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber','height'],None,True,"160")
				config.plugins.hyperioncontrol.framegrabberSize.value = str(width) + "," + str(height)
				setConfigValueFromJson(config.plugins.hyperioncontrol.framegrabberFreq, self.jsonConfigBak, ['framegrabber','frequency_Hz'],1)

				#BlackborderDetector(not in remote only in json)
				bbdEnable = setConfigValueFromJson(config.plugins.hyperioncontrol.bbdEnable, self.jsonConfigBak, ['blackborderdetector','enable'],calculate=None,onlyReturnValue=True)
				if str(bbdEnable).lower() == "true":
					config.plugins.hyperioncontrol.bbdEnable.value = True
				else:
					config.plugins.hyperioncontrol.bbdEnable.value = False
				setConfigValueFromJson(config.plugins.hyperioncontrol.bbdThreshold, self.jsonConfigBak, ['blackborderdetector','threshold'],calculate=100)
				
				self.jsonConfigBak= ""
				
				#reload ConfigList with new values
				self.buildConfig()
				self.session.open(MessageBox,_("The import from the file '/etc/hyperion/hyperion.config.json.bak' has been successfully completed."), MessageBox.TYPE_INFO,timeout=5)
			
			else:
				self.session.open(MessageBox,_("Import failed!\n\nFile '/etc/hyperion/hyperion.config.json.bak' not found!"), MessageBox.TYPE_INFO,timeout=5)

		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,_("Error importing the 'hyperion.config.json.bak'!"), MessageBox.TYPE_INFO,timeout=5)
			self.jsonConfigBak= ""

	def readValuesFromConfig(self):
		
		try:
			#read values from config.json
			self.jsonConfig = ""
			config_file_path ="/etc/hyperion/hyperion.config.json"
			if os.path.exists(config_file_path): 
				valuesFromConfig = open(config_file_path).read()
				lines = valuesFromConfig.split("\n")
				newlines=[]
				for line in lines:
					cleanline = line.replace("\t","")
					if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
						newlines.append(line)
				valuesFromConfig = "\n".join(newlines)
				from collections import OrderedDict
				self.jsonConfig = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
				
				#Framegrabber(not in remote only in json)
				config.plugins.hyperioncontrol.framegrabberSize.value = str(self.jsonConfig['framegrabber']['width']) + "," + str(self.jsonConfig['framegrabber']['height'])
				config.plugins.hyperioncontrol.framegrabberFreq.value = str(int(self.jsonConfig['framegrabber']['frequency_Hz']))
				#BlackborderDetector(not in remote only in json)
				bbdEnable = str(self.jsonConfig['blackborderdetector']['enable'])
				if bbdEnable == "true":
					config.plugins.hyperioncontrol.bbdEnable.value = True
				else:
					config.plugins.hyperioncontrol.bbdEnable.value = False
				config.plugins.hyperioncontrol.bbdThreshold.value = str(int(float(self.jsonConfig['blackborderdetector']['threshold']) * 100))
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,_("Error loading the '/etc/hyperion/hyperion.config.json'!"), MessageBox.TYPE_INFO,timeout=5)

	def buildConfig(self):
		
		self.list = []
		if self.setupPage == "transform":
			self.list.append(getConfigListEntry(_("HSV (0-500, default=100)"), ))
			self.list.append(getConfigListEntry(_("Saturation Gain"), NoSave(config.plugins.hyperioncontrol.saturationGain)))
			self.list.append(getConfigListEntry(_("Value Gain"), NoSave(config.plugins.hyperioncontrol.valueGain)))
			self.list.append(getConfigListEntry("", )) 
			
			self.list.append(getConfigListEntry(_("HSL"), ))
			self.list.append(getConfigListEntry(_("SaturationL Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.saturationLGain)))
			self.list.append(getConfigListEntry(_("Luminance Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.luminanceGain)))
			self.list.append(getConfigListEntry(_("Luminance Minimum (0-50, default=0)"), NoSave(config.plugins.hyperioncontrol.luminanceMinimum)))
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Gamma (0-500, default=250)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.gammaRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.gammaGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.gammaBlue)) )
			self.list.append(getConfigListEntry("", ))
					
			self.list.append(getConfigListEntry(_("Threshold (0-100, default=0)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.thresholdRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.thresholdGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.thresholdBlue)) )
			self.list.append(getConfigListEntry("", ))
					
			self.list.append(getConfigListEntry(_("Temperature (0-255, default=255)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.temperatureRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.temperatureGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.temperatureBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Red Adjust (0-255, default=255, 0, 0)"), ))
			self.list.append(getConfigListEntry(_("red (fix)"), NoSave(config.plugins.hyperioncontrol.redAdjustRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.redAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.redAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Green Adjust (0-255, default=0, 255, 0)"), ))
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.greenAdjustRed)) )
			self.list.append(getConfigListEntry(_("green (fix)"), NoSave(config.plugins.hyperioncontrol.greenAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.greenAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Blue Adjust (0-255, default=0, 0, 255)"), ))
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blueAdjustRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blueAdjustGreen)) )
			self.list.append(getConfigListEntry(_("blue (fix)"), NoSave(config.plugins.hyperioncontrol.blueAdjustBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Whitelevel (0-100, default=100)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.whitelevelRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.whitelevelGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.whitelevelBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Blacklevel (0-50, default=0)"), )) 
			self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blacklevelRed)) )
			self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blacklevelGreen)) )
			self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.blacklevelBlue)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("Framegrabber"), ))
			self.list.append(getConfigListEntry(_("Size (default=160x160)"), NoSave(config.plugins.hyperioncontrol.framegrabberSize)) )
			self.list.append(getConfigListEntry(_("Frequency in Hz (10-25, default=10)"), NoSave(config.plugins.hyperioncontrol.framegrabberFreq)) )
			self.list.append(getConfigListEntry("", ))
			
			self.list.append(getConfigListEntry(_("BlackBorderDetection"), ))
			self.list.append(getConfigListEntry(_("enabled"), NoSave(config.plugins.hyperioncontrol.bbdEnable)) )
			self.list.append(getConfigListEntry(_("threshold (1-20, default=2)"), NoSave(config.plugins.hyperioncontrol.bbdThreshold)) )
			
		elif self.setupPage == "device":
			self.list.append(getConfigListEntry(_("Device-Setup"), ))
			self.list.append(getConfigListEntry(_("Device-Typ"), config.plugins.hyperioncontrol.deviceType))
			self.list.append(getConfigListEntry(_("Config-Name"), config.plugins.hyperioncontrol.configName))
			if config.plugins.hyperioncontrol.deviceType.value == "udpraw":
				self.list.append(getConfigListEntry(_("Device-IP (fix port: 19446)"), config.plugins.hyperioncontrol.outputIP))
			elif config.plugins.hyperioncontrol.deviceType.value == "philipshue":
				self.list.append(getConfigListEntry(_("Device-IP"), config.plugins.hyperioncontrol.outputIP))
			if config.plugins.hyperioncontrol.deviceType.value in ("sedu","adalight","atmo"):
				self.list.append(getConfigListEntry(_("delayafterconnect (0-20, default=0)"), config.plugins.hyperioncontrol.delayafterconnect))
			self.list.append(getConfigListEntry(_("colorOrder"), config.plugins.hyperioncontrol.colorOrder))
			self.list.append(getConfigListEntry(_("Baudrate"), config.plugins.hyperioncontrol.baudrate))
			
			self.list.append(getConfigListEntry("", ))
			self.list.append(getConfigListEntry(_("LED-Setup"), ))
			self.ledSetupIndex = self.list.index((_("LED-Setup"), ))
			self.list.append(getConfigListEntry(_("LED begin"), config.plugins.hyperioncontrol.ledBegin))
			if config.plugins.hyperioncontrol.ledBegin.value != "0":
				self.list.append(getConfigListEntry(_("LED direction"), config.plugins.hyperioncontrol.ledDirection))
				self.list.append(getConfigListEntry(_("Number of LEDs top"), config.plugins.hyperioncontrol.ledTop))
				self.list.append(getConfigListEntry(_("Number of LEDs right"), config.plugins.hyperioncontrol.ledRight))
				self.list.append(getConfigListEntry(_("Number of LEDs left"), config.plugins.hyperioncontrol.ledLeft))
				self.list.append(getConfigListEntry(_("Number of LEDs gap bottom"), config.plugins.hyperioncontrol.ledBottomGap))
				self.list.append(getConfigListEntry(_("Corner LED left top"), config.plugins.hyperioncontrol.ledCornerTopLeft))
				self.list.append(getConfigListEntry(_("Corner LED right top"), config.plugins.hyperioncontrol.ledCornerTopRight))
				self.list.append(getConfigListEntry(_("Corner LED left bottom"), config.plugins.hyperioncontrol.ledCornerBottomLeft))
				self.list.append(getConfigListEntry(_("Corner LED right bottom"), config.plugins.hyperioncontrol.ledCornerBottomRight))
				self.list.append(getConfigListEntry(_("Image Area - horizontal depth (default=8%)"), config.plugins.hyperioncontrol.hPicDeep))
				self.list.append(getConfigListEntry(_("Image Area - vertical depth (default=5%)"), config.plugins.hyperioncontrol.vPicDeep))
				self.writeLedCount()
		
		self["config"].list = self.list
		self["config"].l.setList(self.list) 
	
	def getConfigName(self, subsection, configelement):
		for item in subsection.dict():
			conf = subsection.__getattr__(item)
			if conf == configelement:
				return item

	def writeLedCount(self):
		led_top   = int(config.plugins.hyperioncontrol.ledTop.value)
		led_right = int(config.plugins.hyperioncontrol.ledRight.value)
		led_left  = int(config.plugins.hyperioncontrol.ledLeft.value)
		led_gap  = int(config.plugins.hyperioncontrol.ledBottomGap.value)
		led_count = led_top*2 + led_right + led_left - led_gap
		for ledCornerConfig in (config.plugins.hyperioncontrol.ledCornerTopLeft,config.plugins.hyperioncontrol.ledCornerTopRight,config.plugins.hyperioncontrol.ledCornerBottomLeft,config.plugins.hyperioncontrol.ledCornerBottomRight):
			if ledCornerConfig.value:
				led_count += 1
		self.list[self.ledSetupIndex] = ((_("LED-Setup (Number of LEDs: %s)") % led_count,))
		self["config"].l.invalidateEntry(self.ledSetupIndex)
	
	def changed(self):
		current = self["config"].getCurrent()
		configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])
		print "[HyperionControl] - change:", configName
		
		if configName in ("deviceType","ledBegin"):
			self.buildConfig()
			return
		
		if configName.startswith("led"):
			self.writeLedCount()
		
		cmd = ""
		if configName == "saturationGain":
			value = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
			cmd = "-s %s" % value
		elif configName == "valueGain":
			value = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0
			cmd = "-v %s" % value
		elif configName == "saturationLGain":
			value = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
			cmd = "-u %s" % value
		elif configName == "luminanceGain":
			value = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
			cmd = "-m %s" % value
		elif configName == "luminanceMinimum":
			value = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0
			cmd = "-n %s" % value
		elif configName.startswith("gamma"):
			Red = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
			cmd = "-g '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("threshold"):
			Red = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
			cmd = "-t '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("temperature"):
			Red = int(config.plugins.hyperioncontrol.temperatureRed.value)
			Green = int(config.plugins.hyperioncontrol.temperatureGreen.value)
			Blue = int(config.plugins.hyperioncontrol.temperatureBlue.value)
			cmd = "-Z '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("redAdjust"):
			Red = int(config.plugins.hyperioncontrol.redAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
			cmd = "-R '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("greenAdjust"):
			Red = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
			cmd = "-G '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("blueAdjust"):
			Red = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
			Green = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
			Blue = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
			cmd = "-B '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("whitelevel"):
			Red = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
			cmd = "-w '%s %s %s'" % (Red, Green, Blue)
		elif configName.startswith("blacklevel"):
			Red = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
			Green = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
			Blue = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
			cmd = "-b '%s %s %s'" % (Red, Green, Blue)
			
		if cmd:
			send_CMD(cmd)

	def menu(self):
		list = []
		list.append((_("import complete config from hyperion.config.json.bak"), "importCompleteConfig"))
		list.append((_("import LED-config from hyperion.config.json.ledimport"), "importLEDConfig"))
		list.append((_("save current config-file as hyperion.config.json.bak"), "saveAsBak"))
		self.session.openWithCallback(
			self.menuCallback,
			ChoiceBox, 
			title = _("Hyperion Control - Setup-Options"),
			list = list,
		)

	def menuCallback(self, ret):
		ret = ret and ret[1]
		if ret:
			if ret == "importCompleteConfig":
				self.session.openWithCallback(self.importJsonBak, MessageBox,_("Should import all data from existing 'hyperion.config.json.bak'?\n\nAll setup values ​​will be overwritten with the data from the hyperion.config.json.bak!\n\nThe read values ​​can then be changed via the green button be saved in the current hyperion.config.json."), MessageBox.TYPE_YESNO,default=False)
			elif ret == "importLEDConfig":
				if config.plugins.hyperioncontrol.ledBegin.value != "0":
					self.session.open(MessageBox, _("The LED config import is only possible with the option 'LED start: no'!"), MessageBox.TYPE_INFO)
					return
				else:
					self.session.openWithCallback(self.importLEDConfig, MessageBox,_("Should the LED config data import from exist hyperion.config.json.ledimport ?\n\nAll LED values ​​are taken with the data from the hyperion.config.json.ledimport overwritten!\n\nThe read-in values ​​can then be saved via the green button in the current hyperion.config.json."), MessageBox.TYPE_YESNO,default=False)
			elif ret == "saveAsBak":
				config_bakfile_path ="/etc/hyperion/hyperion.config.json.bak"
				message_txt = _("Should the current config file be backed up as 'hyperion.config.json.bak'?")
				if os.path.exists(config_bakfile_path):
					message_txt += _("\n\nThe already existing 'hyperion.config.json.bak' will be overwritten!")
				self.session.openWithCallback(self.saveConfigAsBak, MessageBox, message_txt, MessageBox.TYPE_YESNO,default=False)

	def saveConfigAsBak(self, ret):
		if ret:
			config_file_path ="/etc/hyperion/hyperion.config.json"
			config_bakfile_path ="/etc/hyperion/hyperion.config.json.bak"
			if os.path.exists(config_file_path):
				os.system('cp %s %s' % (config_file_path, config_bakfile_path))
				print "[HyperionControl] configfile saved as .bak"
				self.session.open(MessageBox,_("The current config file was successful saved as '/etc/hyperion/hyperion.config.json.bak'."), MessageBox.TYPE_INFO,timeout=5)
			else:
				self.session.open(MessageBox,_("Backup as .bak failed!\n\nFile '/etc/hyperion/hyperion.config.json' not found!"), MessageBox.TYPE_INFO,timeout=5)
	
	def importLEDConfig(self, ret):
		if not ret:
			return
		try:
			if not self.jsonConfig:
				self.session.open(MessageBox,_("There are no data from '/etc/hyperion/hyperion.config.json'!\n\nAn import is therefore not possible."), MessageBox.TYPE_INFO,timeout=5)
				return
			
			self.jsonConfigImport= ""
			config_file_path ="/etc/hyperion/hyperion.config.json.ledimport"
			if os.path.exists(config_file_path): 
				#json.bak einlesen
				valuesFromConfig = open(config_file_path).read()
				lines = valuesFromConfig.split("\n")
				newlines=[]
				for line in lines:
					cleanline = line.replace("\t","")
					if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
						newlines.append(line)
				valuesFromConfig = "\n".join(newlines)
				from collections import OrderedDict
				self.jsonConfigImport = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
				
				#LED-key aus import json in json übernehmen
				if "leds" in self.jsonConfigImport:
					self.jsonConfig["leds"] = self.jsonConfigImport["leds"]
					#print "=== LED-Config-Import erfolgreich"
					self.session.open(MessageBox,_("The LED Config import was completed successfully.\n\nWhen saving via the green button, the acquired LED information can be saved in the current config."), MessageBox.TYPE_INFO,timeout=10)
				else:
					self.session.open(MessageBox,_("The LED config import failed. No LED config could be found in the import file!"), MessageBox.TYPE_INFO,timeout=5)
				self.jsonConfigImport= ""
			else:
				self.session.open(MessageBox,_("Import failed!\n\nFile '/etc/hyperion/hyperion.config.json.ledimport' not found!"), MessageBox.TYPE_INFO,timeout=5)
				
		except:
			import traceback
			traceback.print_exc()
			self.session.open(MessageBox,_("Error when importing the LED config!"), MessageBox.TYPE_INFO,timeout=5)
			self.jsonConfigImport= ""

	def key_red(self):
		config.plugins.hyperioncontrol.save()
		self.close()

	def key_green(self):
		self.saveJson()
		
	def key_yellow(self):
		if self.setupPage == "transform":
			self.setupPage = "device"
			self["key_yellow"].setText(_("Transform"))
		else:
			self.setupPage = "transform"
			self["key_yellow"].setText(_("Device/LED"))
		self.buildConfig()
	
	def key_blue(self):
		self.menu()
	
	def key_exit(self):
		config.plugins.hyperioncontrol.save()
		self.close(self.session, "exit")


class hyperionControlSystem(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlSystem" position="center,200" size="900,800" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;30" foregroundColor="white" name="" position="680,720" size="200,70" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlSystem" position="center,80" size="590,620" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel font="Regular;22" foregroundColor="white" name="" position="440,570" size="140,40" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""

	def __init__(self, session):
		Screen.__init__(self, session)
		
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions','SetupActions'], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'previousSection' : self.pageUp,
		'nextSection' : self.pageDown,
		'left'	: self.pageUp,
		'right'	: self.pageDown,
		})

		self.litems = []
		tlf = TemplatedListFonts()
		self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent, margin=ePoint(0,0), selectionZoom=1.3)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
		self['list'] = self.ml	
		self.onLayoutFinish.append(self.buildList)
		self.setTitle(_("Hyperion Control - SystemCTL"))

	def buildList(self):
		self.ml.l.setBuildFunc(self.showList, True)
		self.litems = []
		self.litems.append(((_("start hyperionaml"), "start"),))
		self.litems.append(((_("stop hyperionaml"), "stop"),))
		self.litems.append(((_("restart hyperionaml"), "restart"),))
		self.litems.append(((_("status hyperionaml"), "status"),))
		self.litems.append(((_("start hyperion"), "start_1"),))
		self.litems.append(((_("stop hyperion"), "stop_1"),))
		self.litems.append(((_("restart hyperion"), "restart_1"),))
		self.litems.append(((_("status hyperion"), "status_1"),))
		self.litems.append(((_("disable hyperion"), "disable_1"),))
		self.litems.append(((_("enable autostart"), "enable"),))
		self.litems.append(((_("disable autostart"), "disable"),))
		self.litems.append(((_("status autostart"), "status_2"),))
		self["list"].setList(self.litems)

	def showList(self, item, selected):
		res = [item]
		(name, cmd) = item
		if sz_w == 1920:
			res.append(MultiContentEntryText(pos=(0,0), size=(880,60), font=0, text=name, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		else:
			res.append(MultiContentEntryText(pos=(0,0), size=(575,40), font=0, text=name, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		return res

	def pageUp(self):
		self["list"].pageUp()

	def pageDown(self):
		self["list"].pageDown()

	def key_red(self):
		self.close()

	def key_green(self):
		pass

	def key_yellow(self):
		pass

	def key_blue(self):
		about(self)

	def key_exit(self):
		self.close(self.session, "exit")

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		(name, returnValue) = self['list'].getCurrent()[0]
		if returnValue == "start":
			self.session.open(Console,_("start hyperionaml"),["%s start hyperionaml" % hyperionremote_sh])
		elif returnValue == "stop":
			self.session.open(Console,_("stop hyperionaml"),["%s stop hyperionaml" % hyperionremote_sh])
		elif returnValue == "restart":
			self.session.open(Console,_("restart hyperionaml"),["%s restart hyperionaml" % hyperionremote_sh])
		elif returnValue == "status":
			self.session.open(Console,_("status hyperionaml"),["%s status hyperionaml" % hyperionremote_sh])
		if returnValue == "start_1":
			self.session.open(Console,_("start hyperion"),["%s start hyperion" % hyperionremote_sh])
		elif returnValue == "stop_1":
			self.session.open(Console,_("stop hyperion"),["%s stop hyperion" % hyperionremote_sh])
		elif returnValue == "restart_1":
			self.session.open(Console,_("restart hyperion"),["%s restart hyperion" % hyperionremote_sh])
		elif returnValue == "status_1":
			self.session.open(Console,_("status hyperion"),["%s status hyperion" % hyperionremote_sh])
		elif returnValue == "disable_1":
			self.session.open(Console,_("disable hyperion"),["%s disable hyperion" % hyperionremote_sh])
		elif returnValue == "enable":
			self.session.open(Console,_("enable systemd-hyperionaml.timer"),["%s enable systemd-hyperionaml.timer" % hyperionremote_sh])
		elif returnValue == "disable":
			self.session.open(Console,_("disable systemd-hyperionaml.timer"),["%s disable systemd-hyperionaml.timer" % hyperionremote_sh])
		elif returnValue == "status_2":
			self.session.open(Console,_("status systemd-hyperionaml.timer"),["%s status systemd-hyperionaml.timer" % hyperionremote_sh])

class hyperionControlEffects(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionControlEffects" position="center,200" size="900,800" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
			<eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionControlEffects" position="center,80" size="590,620" >
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
			<ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
			<eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
			<widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
		</screen>"""
		
	def __init__(self, session):
		Screen.__init__(self, session)
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions',], {
		'cancel': self.key_exit,
		'ok'	: self.key_ok,
		'red'	: self.key_red,
		'green'	: self.key_green,
		'yellow': self.key_yellow,
		'blue'	: self.key_blue,
		'previousSection' : self.pageUp,
		'nextSection' : self.pageDown,
		'left'	: self.pageUp,
		'right'	: self.pageDown,
		})
		self.setTitle(_("Hyperion Control - Effects"))
		
		tlf = TemplatedListFonts()
		self.litems = [('Blue mood blobs',), ('Cinema brighten lights',), ('Cinema dim lights',), ('Cold mood blobs',), ('Color traces',), ('Full color mood blobs',), ('Green mood blobs',), ('Knight rider',), ('Police Lights Single',), ('Police Lights Solid',), ('Rainbow mood',), ('Rainbow swirl',), ('Rainbow swirl fast',), ('Random',), ('Red mood blobs',), ('Running dots',), ('Snake',), ('Sparks',), ('Sparks Color',), ('Strobe Raspbmc',), ('Strobe blue',), ('Strobe white',), ('Warm mood blobs',), ('X-Mas',), ('UDP listener',), ('UDP multicast listener',), ('System Shutdown',)]
		self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent, margin=ePoint(0,0), selectionZoom=1.2)
		self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
		self['list'] = self.ml

		self.onLayoutFinish.append(self.buildList)

	def buildList(self):
		self.ml.l.setBuildFunc(self.showList, True)
		self["list"].setList(self.litems)

	def showList(self, item, selected):
		res = [item]
		if sz_w == 1920:
			res.append(MultiContentEntryText(pos=(0,0), size=(880,60), font=0, text=item, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		else:
			res.append(MultiContentEntryText(pos=(0,0), size=(575,40), font=0, text=item, flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
		return res

	def pageUp(self):
		self["list"].pageUp()

	def pageDown(self):
		self["list"].pageDown()

	def key_red(self):
		self.close()

	def key_green(self):
		pass
		
	def key_yellow(self):
		pass
		
	def key_blue(self):
		pass

	def key_ok(self):
		exist = self['list'].getCurrent()
		if exist == None:
			return
		effect = self['list'].getCurrent()[0]
		print effect
		cmd = "--effect '%s'" % effect
		send_CMD(cmd)

	def key_exit(self):
		self.close(self.session, "exit")


class hyperionStart(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="hyperionStart" position="center,center" size="900,650">
			<eLabel font="Regular;42" foregroundColor="white" name="" position="0,20" size="900,110" text="Hyperion Control" valign="center" halign="center" transparent="1" />
			<widget name="logo" position="80,170" size="250,250" />
			<widget name="status" position="420,170" size="450,250" font="Regular;23" foregroundColor="black" backgroundColor="white" />
			<eLabel name="" position="10,550" size="880,2" backgroundColor="grey" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,560" size="210,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="230,560" size="210,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="450,560" size="210,70" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="670,560" size="210,70" scale="stretch" />
			<widget source="key_red" render="Label" backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,560"  size="210,70" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" transparent="1" valign="center" zPosition="1" />
			<widget source="key_green" render="Label" backgroundColor="#1f771f" font="Regular;30" halign="center" position="230,560" size="210,70" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"  transparent="1" valign="center" zPosition="1" />
			<widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="450,560" size="210,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label"  font="Regular;30" foregroundColor="white" position="670,560" size="210,70" valign="center" halign="center" zPosition="1" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="version" font="Regular;30" position="15,500" size="400,40" />
			<ePixmap pixmap="skin_default/buttons/key_menu.png" position="785,490" size="90,45" scale="stretch" />
		</screen>"""
	else:
		skin = """
		<screen name="hyperionStart" position="center,center" size="600,500">
			<eLabel font="Regular;32" foregroundColor="white" name="" position="0,20" size="600,98" text="Hyperion Control" valign="center" halign="center" transparent="1" />
			<widget name="logo" position="40,140" size="210,210" />
			<widget name="status" position="300,160" size="270,180" font="Regular;16" foregroundColor="black" backgroundColor="white" />
			<eLabel name="" position="5,440" size="590,2" backgroundColor="grey" />
			<ePixmap pixmap="skin_default/buttons/red.png" position="5,450" size="145,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="155,450" size="145,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/yellow.png" position="305,450" size="145,40" scale="stretch" />
			<ePixmap pixmap="skin_default/buttons/blue.png" position="455,450" size="145,40" scale="stretch" />
			<widget source="key_red" render="Label" backgroundColor="#9f1313" font="Regular;22" halign="center" position="5,450"  size="145,40" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" transparent="1" valign="center" zPosition="1" />
			<widget source="key_green" render="Label" position="155,450" size="145,40" zPosition="1" font="Regular;22" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
			<widget source="key_yellow" render="Label" font="Regular;22" foregroundColor="white" position="305,450" size="145,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
			<widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="455,450" size="145,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
			<widget name="version" font="Regular;22" position="15,400" size="300,25" />
			<ePixmap pixmap="skin_default/buttons/key_menu.png" position="497,395" size="72,36" />
		</screen>"""
	
	def __init__(self, session):
		Screen.__init__(self, session)
		
		self["logo"] = Pixmap()
		self["version"] = Label(_("Version") + " " + hyperioncontrol_version)
		self["status"] = Label(_("state:"))
		self["key_red"] = StaticText(_("Colors"))
		self["key_green"] = StaticText(_("Effects"))
		self["key_yellow"] = StaticText(_("Setup"))
		self["key_blue"] = StaticText(_("Live Mode"))
		
		self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions','InfobarEPGActions'], 
		{
			'cancel': self.close,
			'red'	: self.key_red,
			'green'	: self.key_green,
			'yellow': self.key_yellow,
			'blue'	: self.key_blue,
			'menu': self.menu,
			'showEventInfo': self.about,
		})
		
		self.onLayoutFinish.append(self.layoutFinished)
		self.onShown.append(self.refreshStatus)
		self.runfirst = True

	def refreshStatus(self):
		status_txt = "\n  " + _("state:") + "\n\n"
		print "=== [HyperionControl] refresh status"
		
		#hyperion-status
		command = Command("systemctl status hyperion")
		command.run(timeout=3)
		cmd_out= command.out
		if "Active: active" in cmd_out:
			status_txt += "  Hyperion: %s\n" % _("active")
		else:
			status_txt += "  Hyperion: %s\n" % _("inactive")
		
		#hyperionaml-status
		command = Command("systemctl status hyperionaml")
		command.run(timeout=3)
		cmd_out= command.out
		if "Active: active" in cmd_out:
			status_txt += "  Hyperionaml: %s\n" % _("active")
		else:
			status_txt += "  Hyperionaml: %s\n" % _("inactive")
		
		#systemd-hyperionaml.timer
		command = Command("systemctl status systemd-hyperionaml.timer")
		command.run(timeout=3)
		cmd_out= command.out
		if "enabled;" in cmd_out:
			status_txt += "  Autostart: %s\n\n" % _("active")
		else:
			status_txt += "  Autostart: %s\n\n" % _("inactive")
		
		#Modus
		self.jsonRemote = readValuesFromRemote(self, self.runfirst, False)
		self.runfirst=False
		if self.jsonRemote:
			if self.jsonRemote["activeEffects"] != []:
				status_txt += "  " + _("Mode: ") + _("Effects") + "\n"
			elif self.jsonRemote["activeLedColor"] != []:
				status_txt += "  " + _("Mode: ") + _("Colors") + "\n"
			else:
				status_txt += "  " + _("Mode: ") + _("Live Mode") + " " + _("or") + " " + _("Image Mode") + "\n"
		self["status"].setText(status_txt)
	
	def layoutFinished(self):
		self["logo"].instance.setPixmapFromFile("/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/hyperion_logo_rund.png")
		self.setTitle(_("Hyperion Control"))

	def about(self):
		about(self)

	def key_red(self):
		self.session.open(hyperionControlColors)

	def key_green(self):
		self.session.open(hyperionControlEffects)
		
	def key_yellow(self):
		self.session.open(hyperionControlSetup)

	def key_blue(self):
		if self["key_blue"].getText():
			send_CMD('--clearall')
			self.session.open(MessageBox,_("The Live Mode has been activated."), MessageBox.TYPE_INFO,timeout=5)
	
	def menu(self):
		self.session.open(hyperionControlSystem)

class HyperionControlPictureBrowser(Screen):
	if sz_w == 1920:
		skin = """
		<screen name="HyperionControlPictureBrowser" position="center,170" size="1500,740" title="Hyperion Control Picture Browser">
			<ePixmap pixmap="Default-FHD/skin_default/buttons/red.svg" position="10,5" size="300,70" />
			<ePixmap pixmap="Default-FHD/skin_default/buttons/green.svg" position="310,5" size="300,70" />
			<widget backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_red" transparent="1" valign="center" zPosition="1" />
			<widget backgroundColor="#1f771f" font="Regular;30" halign="center" position="310,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_green" transparent="1" valign="center" zPosition="1" />
			<eLabel backgroundColor="grey" position="10,80" size="900,1" />
			<eLabel backgroundColor="grey" position="920,5" size="1,725" />
			<widget name="picpreview" position="960,90" size="500,500" />
			<widget enableWrapAround="1" name="filelist" position="10,90" scrollbarMode="showOnDemand" size="900,630" />
		</screen>"""
	else:
		skin = """
		<screen name="HyperionControlPictureBrowser" position="center,120" size="920,430" title="Hyperion Control Picture Browser">
			<ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40"/>
			<ePixmap pixmap="skin_default/buttons/green.png" position="210,5" size="200,40"/>
			<widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
			<widget source="key_green" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
			<eLabel position="10,50" size="590,1" backgroundColor="grey"/>
			<eLabel position="610,5" size="1,420" backgroundColor="grey"/>
			<widget name="picpreview" position="620,80" size="290,290" />
			<widget name="filelist" position="10,60" size="590,360" enableWrapAround="1" scrollbarMode="showOnDemand"/>
		</screen>"""

	def __init__(self, session, current_path=None):
		Screen.__init__(self, session)
		if current_path:
			currDir = current_path
			if not os.path.exists(currDir):
				currDir = "/"
		else:
			currDir = None #start with mountpoints-list

		self.filelist = FileList(currDir, matchingPattern="(?i)^.*\.(jpg|png)")
		self["filelist"] = self.filelist
		self["picpreview"] = Pixmap()
		self["filelist"].onSelectionChanged.append(self.selectionChanged)
		
		self["FilelistActions"] = ActionMap(["SetupActions"],
			{
				"save": self.ok,
				"ok": self.ok,
				"cancel": self.exit
			})
		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))
		
		self.ThumbTimer = eTimer()
		self.ThumbTimer_conn = self.ThumbTimer.timeout.connect(self.showThumb)
		
		self.onShown.append(self.setWindowTitle)

	def setWindowTitle(self):
		self.setTitle(_("Hyperion Control Picture Browser"))

	def ok(self):
		if self["filelist"].canDescent(): # isDir
			self["filelist"].descent()
		else:
			file_name = self["filelist"].getFilename()
			path = self["filelist"].getCurrentDirectory()
			self.exit(path + file_name)

	def showThumb(self):
		if not self["filelist"].canDescent():
			file_name = self["filelist"].getFilename()
			path = self["filelist"].getCurrentDirectory()
			#print "== picname", path + file_name
			self["picpreview"].instance.setPixmapFromFile(path + file_name)
			self["picpreview"].show()

	def selectionChanged(self):
		if self["picpreview"] is None:
			return
		
		if not self["filelist"].canDescent():
			self.ThumbTimer.start(400, True)
		else:
			self["picpreview"].hide()

	def exit(self, retvalue=None):
		self.ThumbTimer.stop()
		self.ThumbTimer_conn = None
		if self.filelist.getCurrentDirectory() is None:
			config.plugins.hyperioncontrol.lastpicpath.value = "/"
		else:
			config.plugins.hyperioncontrol.lastpicpath.value = self["filelist"].getCurrentDirectory()
		config.plugins.hyperioncontrol.save()
		
		self["picpreview"] = None
		self.close(retvalue)

def about(self):
		title="HyperionControl version %s \nplugin by\neinfall, pclin & Sven_H" % hyperioncontrol_version
		self.session.open(MessageBox,("%s") % (title),  MessageBox.TYPE_INFO)

def send_CMD(cmd):
	raw = "hyperion-remote %s" % cmd
	print "[HyperionControl] SEND CMD TO Hyperion Remote: %s" % raw
	os.system(raw)

def setConfigValueFromJson(config, jsonData, jsonKey, calculate=None, onlyReturnValue=False, default=None):
	try:
		for item in jsonKey:
			jsonData = jsonData[item]
		if calculate:
			jsonData = int(jsonData * calculate)
		if onlyReturnValue:
			print "=== [HyperionControl] return value", item, str(jsonData)
			return str(jsonData)
		else:
			config.value = str(jsonData)
			print "=== [HyperionControl] set value to", item, str(jsonData)
	except (KeyError, TypeError, IndexError):
		if onlyReturnValue:
			print "=== [HyperionControl] error on item - return default", item, config.default
			if default:
				return default
			else:
				return config.default
		else:
			config.value = config.default
			print "=== [HyperionControl] error on item - set default", item, config.default

def readValuesFromRemote(self, showMessage=True, setConfig=True):
	print "=== [HyperionControl] readValuesFromRemote"
	try:
		command = Command("hyperion-remote -l")
		command.run(timeout=5)
		
		#read real settings from hyperion-remote -l
		if command.timeout:
			print "[HyperionControl] timeout on readValuesFromRemote"
			if showMessage:
				self.session.open(MessageBox,_("Timeout on load remote-Data!\n\n"), MessageBox.TYPE_INFO,timeout=5)
			return None
		elif command.process.returncode != 0:
			print "[HyperionControl] error on readValuesFromRemote:", str(command.process.returncode), str(command.err)
			if showMessage:
				self.session.open(MessageBox,_("Error on load remote-Data!\n\n") + str(command.err), MessageBox.TYPE_INFO,timeout=5)
			return None
		
		valuesFromRemote = command.out
		
		pos = valuesFromRemote.find("Server info:")
		if pos:
			valuesFromRemote = valuesFromRemote[pos+12:]
			jsonRemote = json.loads(valuesFromRemote)
			if setConfig:
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, jsonRemote, ['transform',0,'saturationGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, jsonRemote, ['transform',0,'valueGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, jsonRemote, ['transform',0,'saturationLGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, jsonRemote, ['transform',0,'luminanceGain'],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, jsonRemote, ['transform',0,'luminanceMinimum'],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, jsonRemote, ['transform',0,'gamma',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, jsonRemote, ['transform',0,'gamma',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, jsonRemote, ['transform',0,'gamma',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, jsonRemote, ['transform',0,'threshold',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, jsonRemote, ['transform',0,'threshold',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, jsonRemote, ['transform',0,'threshold',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, jsonRemote, ['adjustment',0,'redAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, jsonRemote, ['adjustment',0,'redAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, jsonRemote, ['adjustment',0,'redAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, jsonRemote, ['adjustment',0,'greenAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, jsonRemote, ['adjustment',0,'greenAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, jsonRemote, ['adjustment',0,'greenAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, jsonRemote, ['adjustment',0,'blueAdjust',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, jsonRemote, ['adjustment',0,'blueAdjust',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, jsonRemote, ['adjustment',0,'blueAdjust',2])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, jsonRemote, ['temperature',0,'correctionValues',0])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, jsonRemote, ['temperature',0,'correctionValues',1])
				setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, jsonRemote, ['temperature',0,'correctionValues',2])
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, jsonRemote, ['transform',0,'blacklevel',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, jsonRemote, ['transform',0,'blacklevel',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, jsonRemote, ['transform',0,'blacklevel',2],100)
				
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, jsonRemote, ['transform',0,'whitelevel',0],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, jsonRemote, ['transform',0,'whitelevel',1],100)
				setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, jsonRemote, ['transform',0,'whitelevel',2],100)
			
			return jsonRemote

	except:
		import traceback, sys
		traceback.print_exc()
		exc_type, exc_value, exc_traceback = sys.exc_info()
		if showMessage:
			error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
			self.session.open(MessageBox,_("Error on load remote-Data!\n\nError:\n%s") % error, MessageBox.TYPE_INFO,timeout=10)
		return None

